
-- file: adc_interface_tb.vhd
--
-- DISCLAIMER OF LIABILITY
--
-- This file contains proprietary and confidential information of
-- Xilinx, Inc. ("Xilinx"), that is distributed under a license
-- from Xilinx, and may be used, copied and/or disclosed only
-- pursuant to the terms of a valid license agreement with Xilinx.
--
-- XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION
-- ("MATERIALS") "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
-- EXPRESSED, IMPLIED, OR STATUTORY, INCLUDING WITHOUT
-- LIMITATION, ANY WARRANTY WITH RESPECT TO NONINFRINGEMENT,
-- MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE. Xilinx
-- does not warrant that functions included in the Materials will
-- meet the requirements of Licensee, or that the operation of the
-- Materials will be uninterrupted or error-free, or that defects
-- in the Materials will be corrected. Furthermore, Xilinx does
-- not warrant or make any representations regarding use, or the
-- results of the use, of the Materials in terms of correctness,
-- accuracy, reliability or otherwise.
--
-- Xilinx products are not designed or intended to be fail-safe,
-- or for use in any application requiring fail-safe performance,
-- such as life-support or safety devices or systems, Class III
-- medical devices, nuclear facilities, applications related to
-- the deployment of airbags, or any other applications that could
-- lead to death, personal injury or severe property or
-- environmental damage (individually and collectively, "critical
-- applications"). Customer assumes the sole risk and liability
-- of any use of Xilinx products in critical applications,
-- subject only to applicable laws and regulations governing
-- limitations on product liability.
--
-- Copyright 2009 Xilinx, Inc.
-- All rights reserved.
--
-- This disclaimer and copyright notice must be retained as part
-- of this file at all times.
--

------------------------------------------------------------------------------
-- SelectIO wizard demonstration testbench
------------------------------------------------------------------------------
-- This demonstration testbench instantiates the example design for the 
--   SelectIO wizard. 
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;
use ieee.std_logic_textio.all;

library std;
use std.textio.all;

library work;
use work.all;

entity adc_interface_tb is
end adc_interface_tb;

architecture test of adc_interface_tb is

component adc_interface_exdes
generic (
  -- Clock -> q modeling delay
  TCQ        : time    := 100 ps;
  -- width of the data for the system
  sys_w      : integer := 6;
  -- width of the data for the device
  dev_w      : integer := 12;
  -- width of the address for the memory
  add_w      : integer := 10;
  -- depth of the memory
  add_l      : integer := 1024);
port (
  -- Memory interface
  ADDRESS_USER             : in    std_logic_vector(add_w-1 downto 0);
  DATA_IN_USER             : in    std_logic_vector(dev_w-1 downto 0);
  DATA_OUT_USER            : out   std_logic_vector(dev_w-1 downto 0);
  ENABLE_USER              : in    std_logic;
  WRITE_USER               : in    std_logic;
  ENABLE_DEVICE            : in    std_logic;
  -- From the system into the device
  DATA_IN_FROM_PINS_P      : in    std_logic_vector(sys_w-1 downto 0);
  DATA_IN_FROM_PINS_N      : in    std_logic_vector(sys_w-1 downto 0);
  CLK_IN_P                 : in    std_logic;
  CLK_IN_N                 : in    std_logic;
  CLK_RESET                : in    std_logic;
  IO_RESET                 : in    std_logic);
end component;
  constant TCQ             : time    := 100 ps;
  constant clk_per         : time    :=  10 ns; -- 100 MHz clk
  constant sys_w           : integer := 6;
  constant dev_w           : integer := 12;
  constant add_w           : integer := 10;
  constant data_rate       : integer := 2;
  signal   ADDRESS_USER    : std_logic_vector(add_w-1 downto 0);
  signal   DATA_IN_USER    : std_logic_vector(dev_w-1 downto 0) := (others => '0');
  signal   DATA_OUT_USER   : std_logic_vector(dev_w-1 downto 0);
  signal   ENABLE_USER     : std_logic;
  signal   WRITE_USER      : std_logic;
  signal   ENABLE_DEVICE   : std_logic;
  -- From the system into the device
  signal   DATA_IN_FROM_PINS   : std_logic_vector(sys_w-1 downto 0);
  signal   DATA_IN_FROM_PINS_P : std_logic_vector(sys_w-1 downto 0);
  signal   DATA_IN_FROM_PINS_N : std_logic_vector(sys_w-1 downto 0);
  signal   CLK_IN             : std_logic := '0';
  signal   CLK_IN_P           : std_logic;
  signal   CLK_IN_N           : std_logic;
  signal   CLK_RESET          : std_logic;
  signal   IO_RESET           : std_logic;


begin

  -- Any aliases
   CLK_IN_P <= CLK_IN;
   CLK_IN_N <= not CLK_IN;

   DATA_IN_FROM_PINS_P <=  DATA_IN_FROM_PINS;
   DATA_IN_FROM_PINS_N <=  not (DATA_IN_FROM_PINS);

  -- clock generator- 100 MHz simulation clock
  --------------------------------------------
  process begin
    CLK_IN <= not CLK_IN; wait for (clk_per/2);
  end process;

  -- Test sequence
  process begin
    DATA_IN_FROM_PINS <= (others => '0');
    -- reset the logic
    CLK_RESET   <= '1';
    IO_RESET    <= '1';
    CLK_RESET   <= '0';
    wait for (5*clk_per);
    WRITE_USER    <= '0';
    ENABLE_USER   <= '0';
    -- start up the io
    IO_RESET      <= '0';
    wait for (8*clk_per);
    ENABLE_DEVICE <= '1';
    -- Drive data onto the pins
    DATA_IN_FROM_PINS <= (others => '0');
    for ii in 0 to 255 loop
      ADDRESS_USER <= conv_std_logic_vector(ii, add_w);
      DATA_IN_FROM_PINS <= conv_std_logic_vector(ii, sys_w);
      wait for (clk_per/data_rate);
    end loop;
    report "Simulation Stopped." severity failure;
    wait;
  end process;

  -- Instantiation of the example design

  dut : adc_interface_exdes
  generic map
  (TCQ   => 100 ps,
   sys_w => 6,
   dev_w => 12,
   add_w => 10)
  port map
  (--Memory interface
   ADDRESS_USER              => ADDRESS_USER,
   DATA_IN_USER              => DATA_IN_USER,
   DATA_OUT_USER             => DATA_OUT_USER,
   ENABLE_USER               => ENABLE_USER,
   WRITE_USER                => WRITE_USER,
   ENABLE_DEVICE             => ENABLE_DEVICE,
   -- From the system into the device
   DATA_IN_FROM_PINS_P       => DATA_IN_FROM_PINS_P,
   DATA_IN_FROM_PINS_N       => DATA_IN_FROM_PINS_N,
   CLK_IN_P                  => CLK_IN_P,
   CLK_IN_N                  => CLK_IN_N,
   CLK_RESET                 => CLK_RESET,
   IO_RESET                  => IO_RESET);
end test;



