----------------------------------------------------------------------------------
-- Company: 
-- Engineer: Istvan Nagy, buenos@freemail.hu
-- 
-- Create Date:    15:58:13 05/30/2010 
-- Design Name: 
-- Module Name:    s6bfip_adc - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
-- This module handles the incoming data from the ADC (Texas Instruments ADS5517),
--controls the bahaviour of both the ADC and the PGA.
--The ADC data is continously packed into 32bit dwords, but only the upper 8 bits,
--then sento to the wishbone bus to write it into the memory. This starts when we
--set the adc_start flag, and finishes when 64MBytes has been written. 1 transfer
--at every clock edge, at 50MHz.
--PGA gain: see the PGA870 datasheet. 0=-11.5dB, 63=+20dB
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity s6bfip_adc is
    Port ( --PORT LIST:------------------
			 --FPGA_PINS (EXTERNAL):---
			  ADC_CLK_REFOUT_P : out    std_logic; --set LVDS+Rs --200MHz
			  ADC_CLK_REFOUT_N  : out    std_logic; --set LVDS+Rs			 
			  ADC_CLK_IN_P   : in    std_logic; --set LVDS+Rp, use GCLK-pin
			  ADC_CLK_IN_N   : in    std_logic; --set LVDS+Rp, use GCLK-pin
			  ADC_DATA_IN_P   : in    std_logic_vector(5 downto 0); --set LVDS+Rp
			  ADC_DATA_IN_N   : in    std_logic_vector(5 downto 0); --set LVDS+Rp
			  ADC_OE : out std_logic;
			  ADC_OVR : in std_logic;
			  ADC_SDATA : out std_logic;
			  ADC_SCLK : out std_logic;
			  ADC_SEN : out std_logic;
			  ADC_RESET : out std_logic;
			  PGA_DATA_out  : out    std_logic_vector(5 downto 0); 
			  PGA_STROBE : out std_logic;
			 --DATA BUS (wishbone):
			 adcif_wb_data_o : out std_logic_vector(31 downto 0); 
			 adcif_wb_data_i : in std_logic_vector(31 downto 0);
			 adcif_wb_addr_o : out std_logic_vector(25 downto 0);
			 adcif_wb_cyc_o : out std_logic;
			 adcif_wb_stb_o : out std_logic;
			 adcif_wb_wr_o : out std_logic;
			 adcif_wb_ack_i : in std_logic;
			 adcif_wb_clk_o : out std_logic; --50MHz
			 adcif_wb_sel_o : out std_logic_vector(3 downto 0);
			 --CONTROL:
			 adc_overshoot_detected : out std_logic;
			 pga_gain : in std_logic_vector(5 downto 0); 
			 adc_triggerlevel : in std_logic_vector(7 downto 0); 
			 adc_start : in  std_logic; --edge sensitive
			 adc_finished : out  std_logic;
			 --SYSTEM:
			 x25m_clkin : in std_logic;
			 adcif_reset : in std_logic --active high
			 );
end s6bfip_adc;


architecture Behavioral of s6bfip_adc is

   -- Internal Signals ------------------------------------------------------------
	--SIGNAL dummy : std_logic_vector(15 downto 0);	--write data bus
	SIGNAL adc_localreset : std_logic;
	SIGNAL CLK_OUT200M : std_logic;
	SIGNAL LOCKED : std_logic;
	SIGNAL local_adc_parallel_clk : std_logic;
	SIGNAL adc_byteposition : std_logic_vector(1 downto 0); 
	SIGNAL adc_32bit_data : std_logic_vector(31 downto 0); 
	SIGNAL adc_32bit_data_shaddow : std_logic_vector(31 downto 0); 
	SIGNAL DATA_IN_TO_DEVICE : std_logic_vector(11 downto 0); 
	SIGNAL adcif_wb_addr_o_local : std_logic_vector(25 downto 0); 
	SIGNAL adc_wb_run  : std_logic;
	SIGNAL adcif_wb_clk_o_local : std_logic;



	-- COMPONENT DECLARATIONS (introducing the IPs) --------------------------------

	component adc_interface
	generic
	 (-- width of the data for the system
	  sys_w       : integer := 6;
	  -- width of the data for the device
	  dev_w       : integer := 12);
	port
	 (
	  -- From the system into the device
	  DATA_IN_FROM_PINS_P     : in    std_logic_vector(sys_w-1 downto 0);
	  DATA_IN_FROM_PINS_N     : in    std_logic_vector(sys_w-1 downto 0);
	  DATA_IN_TO_DEVICE       : out   std_logic_vector(dev_w-1 downto 0);
	  CLK_IN_P                : in    std_logic;
	  CLK_IN_N                : in    std_logic;
	  CLK_OUT                 : out   std_logic;
	  CLK_RESET               : in    std_logic;
	  IO_RESET                : in    std_logic);
	end component;

	component pll_for_adc
	port
	 (-- Clock in ports
	  CLK_IN1           : in     std_logic;
	  -- Clock out ports
	  CLK_OUT1          : out    std_logic;
	  -- Status and control signals
	  RESET             : in     std_logic;
	  LOCKED            : out    std_logic
	 );
	end component;


-- --------ARCHITECTURE BODY BEGINS -----------------------------------------------
begin

	adcif_wb_sel_o <= "1111"; --always write 32bits


	-- COMPONENT INSTALLATIONS (connecting the IPs to local signals) ---------------
	inst_adc : adc_interface
	  port map
		(
			  -- From the system into the device
		 DATA_IN_FROM_PINS_P     => ADC_DATA_IN_P,     -- IN     [5:0]
		 DATA_IN_FROM_PINS_N     => ADC_DATA_IN_N,     -- IN     [5:0]
		 DATA_IN_TO_DEVICE       => DATA_IN_TO_DEVICE,       -- OUT    [11:0]
		 CLK_IN_P                => ADC_CLK_IN_P,                -- IN
		 CLK_IN_N                => ADC_CLK_IN_N,                -- IN
		 CLK_OUT                 => local_adc_parallel_clk,        -- OUT
		 CLK_RESET               => adc_localreset,               -- IN
		 IO_RESET                => adc_localreset);               -- IN


	--PLL to generate 200MHz low-jitter ADC clock from the 25MHz osc clock
	inst_pll_for_adc : pll_for_adc
	  port map
		(-- Clock in ports
		 CLK_IN1            => x25m_clkin,
		 -- Clock out ports
		 CLK_OUT1           => CLK_OUT200M,
		 -- Status and control signals
		 RESET              => adcif_reset,
		 LOCKED             => LOCKED);

   --ADC ref clock output buffer:
	OBUFDS_inst : OBUFDS
   generic map (
      IOSTANDARD => "LVDS_33")
   port map (
      O => ADC_CLK_REFOUT_P,     -- Diff_p output (connect directly to top-level port)
      OB => ADC_CLK_REFOUT_N,   -- Diff_n output (connect directly to top-level port)
      I => CLK_OUT200M      -- Buffer input 
		);


	-- MAIN LOGIC: -----------------------------------------------------------------
	
	PGA_DATA_out <= pga_gain;
	PGA_STROBE <= '1';
	adc_overshoot_detected <= ADC_OVR;
	ADC_OE <= '1';
	
	--the serial config interface is not used:
	ADC_SDATA <= '1';
	ADC_SCLK <= '1';
	ADC_SEN <= '1';

	--RESET TO ADC:
	ADC_RESET <= (NOT LOCKED) or adcif_reset;
	adc_localreset <= (NOT LOCKED) or adcif_reset;
	
	--DATA packing to 32bits, and wb clock generation/synchronization:
    process (adc_localreset, local_adc_parallel_clk, adc_byteposition)
    begin
       if (adc_localreset='1') then
           adc_32bit_data <= "00000000000000000000000000000000";
			  adc_32bit_data_shaddow <= "00000000000000000000000000000000";
			  adc_byteposition <= "00";
			  adcif_wb_clk_o <='0';
			  adcif_wb_clk_o_local <='0';
       elsif (local_adc_parallel_clk'event and local_adc_parallel_clk='1') then
         adc_byteposition <= adc_byteposition +1;
			if (adc_byteposition = "00") then
			  adc_32bit_data_shaddow <= adc_32bit_data; --latch it every 4 clk cycles
			  adc_32bit_data(7 downto 0) <= DATA_IN_TO_DEVICE (11 downto 4);
			  adcif_wb_clk_o <='0';
			  adcif_wb_clk_o_local <='0';
			elsif (adc_byteposition= "01") then
			  adc_32bit_data(15 downto 8) <= DATA_IN_TO_DEVICE (11 downto 4);
			  adcif_wb_clk_o <='0';
			  adcif_wb_clk_o_local <='0';
			elsif (adc_byteposition= "10") then
			  adc_32bit_data(23 downto 16) <= DATA_IN_TO_DEVICE (11 downto 4);
			  adcif_wb_clk_o <='1';
			  adcif_wb_clk_o_local <='1';
			else --(adc_byteposition="11")
			  adc_32bit_data(31 downto 24) <= DATA_IN_TO_DEVICE (11 downto 4);
			  adcif_wb_clk_o <='1';
			  adcif_wb_clk_o_local <='1';
			end if;
       end if;
    end process;	
	 
	 
	 --wishbone bus interface:
	 adcif_wb_data_o <= adc_32bit_data_shaddow;
	 --control signal scheduling:
	 --1 transfer/clock at 50MHz
	 process ( adc_localreset, adc_wb_run, adcif_wb_clk_o_local, adcif_wb_addr_o_local)
    begin
       if (adc_localreset='1') then
           adcif_wb_addr_o <= "00000000000000000000000000";
			  adcif_wb_addr_o_local <= "00000000000000000000000000";
			  adcif_wb_cyc_o <= '0';
			  adcif_wb_stb_o <= '0';
			  adcif_wb_wr_o <= '0';
       elsif (adcif_wb_clk_o_local'event and adcif_wb_clk_o_local='1') then
			if (adc_wb_run='1') then 
			--if (adc_wb_run='1' and DATA_IN_TO_DEVICE (11 downto 6) = adc_triggerlevel(7 downto 2)) then
           adcif_wb_addr_o <= adcif_wb_addr_o_local +1;
			  adcif_wb_addr_o_local <= adcif_wb_addr_o_local +1;
			  adcif_wb_cyc_o <= '1';
			  adcif_wb_stb_o <= '1';
			  adcif_wb_wr_o <= '1';			
			else
           adcif_wb_addr_o <= "00000000000000000000000000";
			  adcif_wb_addr_o_local <= "00000000000000000000000000";
			  adcif_wb_cyc_o <= '0';
			  adcif_wb_stb_o <= '0';
			  adcif_wb_wr_o <= '0';			
			end if;
       end if;
    end process;

	--run flipflop:
	 process ( adc_localreset, adcif_wb_clk_o_local, adc_start, adcif_wb_addr_o_local)
    begin
       if (adc_localreset='1') then
           adc_wb_run <= '0';
       elsif (adcif_wb_clk_o_local'event and adcif_wb_clk_o_local='1') then
         --start upon user request:
			if (adc_start='1') then
			  adc_wb_run <= '1';
			  adc_finished <= '0';
			--stop after 64MBytes stored:
			elsif (adcif_wb_addr_o_local = "11111111111111111111111111") then
			  adc_wb_run <= '0';
			  adc_finished <= '1';
			end if;
       end if;
    end process;	




-- -------- END OF FILE -----------------------------------------------------------
end Behavioral;
