----------------------------------------------------------------------------------
-- Company: 
-- Engineer: Istvan Nagy, buenos@freemail.hu
-- 
-- Create Date:    15:58:13 05/30/2010 
-- Design Name:  interface to an ADSP-BF544 Digital Signal Processor
-- Module Name:    s6bfip_dsp - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
-- Access address regions:
--  BANK3: instruction buffer for FIFO boot, 64kbytes buffer
--  BANK1: onboard DRAM memory access, 64Mbytes window
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity s6bfip_dsp is 
    Port ( --FPGA PINS (EXTERNAL):
			  dsp_reset_n : out  STD_LOGIC;
			  dsp_refclk :  out  STD_LOGIC;	
			  dsp_ioclk: in  STD_LOGIC;  --to a GCLK pin!
			  dsp_bmode : out  STD_LOGIC_VECTOR (3 downto 0);
			  dsp_dmar1_n : out  STD_LOGIC;
			  dsp_hwait : in  STD_LOGIC;
			  dsp_ams_n : in  STD_LOGIC_VECTOR (3 downto 0);
			  dsp_data : inout  STD_LOGIC_VECTOR (15 downto 0);
			  dsp_abe_n :in  STD_LOGIC_VECTOR (1 downto 0);
			  dsp_awe_n : in  STD_LOGIC;  --to a GCLK pin!
			  dsp_aoe_n : in  STD_LOGIC;
			  dsp_are_n : in  STD_LOGIC;  --to a GCLK pin!
			  dsp_ardy : out  STD_LOGIC;
			  dsp_addr : in STD_LOGIC_VECTOR (25 downto 1);
			  dsp_flag_from_dsp : in  STD_LOGIC_VECTOR (1 downto 0);
			  dsp_flag_to_dsp : out  STD_LOGIC_VECTOR (1 downto 0);
			  dsp_uart_tx : out  STD_LOGIC;
			  dsp_uart_rx : in  STD_LOGIC;
			  dsp_interrupt : out  STD_LOGIC; --to a GPIO pin
			  dsp_nmi_n : out  STD_LOGIC;
			  dsp_hostif_data :inout  STD_LOGIC_VECTOR (7 downto 0);
			  dsp_hostif_ce_n : out  STD_LOGIC;
			  dsp_hostif_rd_n : out  STD_LOGIC;
			  dsp_hostif_wr_n : out  STD_LOGIC;
			  dsp_hostif_addr : out  STD_LOGIC_VECTOR (1 downto 0); --0==data, 1=config
			  dsp_hostif_ack : in  STD_LOGIC;
			  --DATA BUS (wishbone):
			  dspif_wb_data_o : out std_logic_vector(31 downto 0); 
			  dspif_wb_data_i : in std_logic_vector(31 downto 0);
			  dspif_wb_addr_o : out std_logic_vector(25 downto 0);
			  dspif_wb_cyc_o : out std_logic;
			  dspif_wb_stb_o : out std_logic;
			  dspif_wb_wr_o : out std_logic;
			  dspif_wb_ack_i : in std_logic;
			  dspif_wb_clk_o : out std_logic; 
			  dspif_wb_sel_o : out std_logic_vector(3 downto 0);
			  --instruction buffer :
			  dsp_instr_wb_data_o : in std_logic_vector(31 downto 0); 
			  dsp_instr_wb_data_i : out std_logic_vector(31 downto 0);
			  dsp_instr_wb_addr_o : in std_logic_vector(15 downto 0);
			  dsp_instr_wb_cyc_o : in std_logic;
			  dsp_instr_wb_stb_o : in std_logic;
			  dsp_instr_wb_wr_o : in std_logic;
			  dsp_instr_wb_ack_i : out std_logic;
			  dsp_instr_wb_clk_o : in std_logic; 
			  dsp_instr_wb_sel_o :  in std_logic_vector(3 downto 0);	
			  --Control:
			  dsp_start_boot_frompcie : in std_logic;
			  dsp_reset_control_frompcie : in std_logic;
			  dsp_irq_frompcie : in std_logic; 
			  dsp_nmi_frompcie : in std_logic; 
			  dsp_flag_to_pcie : out  STD_LOGIC_VECTOR (1 downto 0);
			  dsp_flag_from_pcie : in  STD_LOGIC_VECTOR (1 downto 0);			  
			  --System:
			  x25m_clkin : in std_logic;
			  dspif_reset : in std_logic --active high			  
			  );
end s6bfip_dsp;

architecture Behavioral of s6bfip_dsp is
   -- Internal Signals ------------------------------------------------------------
	--SIGNAL dummy : std_logic_vector(15 downto 0);	--write data bus

	--instruction buffer ports:
	SIGNAL clka:  std_logic;
	SIGNAL ena:  std_logic;
	SIGNAL wea:  std_logic_VECTOR(0 downto 0);
	SIGNAL addra:  std_logic_VECTOR(13 downto 0);
	SIGNAL dina:  std_logic_VECTOR(31 downto 0);
	SIGNAL douta:  std_logic_VECTOR(31 downto 0);
	SIGNAL clkb:  std_logic;
	SIGNAL enb:  std_logic;
	SIGNAL web:  std_logic_VECTOR(0 downto 0);
	SIGNAL addrb:  std_logic_VECTOR(14 downto 0);
	SIGNAL dinb:  std_logic_VECTOR(15 downto 0);
	SIGNAL doutb:  std_logic_VECTOR(15 downto 0);
	
	--other signals:
	SIGNAL dsp_fifo_address_counter : STD_LOGIC_VECTOR (15 downto 0);
	SIGNAL dsp_address_latched : STD_LOGIC_VECTOR (25 downto 0);
	SIGNAL dsp_fifo_read_data : STD_LOGIC_VECTOR (15 downto 0);
	SIGNAL async_read_data : STD_LOGIC_VECTOR (15 downto 0);
	SIGNAL async_write_data : STD_LOGIC_VECTOR (15 downto 0);
	SIGNAL dsp_read_data_ready : STD_LOGIC; 	
	SIGNAL dsp_writes : STD_LOGIC; 
	SIGNAL dsp_reads : STD_LOGIC; 
	SIGNAL dsp_aoe_n_previous : STD_LOGIC; 
	SIGNAL wea_vectorize  : STD_LOGIC_VECTOR (0 downto 0);
	SIGNAL dsp_upperlower_vectorize : STD_LOGIC_VECTOR (1 downto 0);
	SIGNAL dsp_upperlower : STD_LOGIC; 
	SIGNAL wb_state : STD_LOGIC_VECTOR (2 downto 0);

	-- COMPONENT DECLARATIONS (introducing the IPs) --------------------------------

	--instruction buffer: 64kbytes. port-A=32bit, port-B=16bit
	component dsp_instr_buffer
		port (
		clka: IN std_logic;
		ena: IN std_logic;
		wea: IN std_logic_VECTOR(0 downto 0);
		addra: IN std_logic_VECTOR(13 downto 0);
		dina: IN std_logic_VECTOR(31 downto 0);
		douta: OUT std_logic_VECTOR(31 downto 0);
		clkb: IN std_logic;
		enb: IN std_logic;
		web: IN std_logic_VECTOR(0 downto 0);
		addrb: IN std_logic_VECTOR(14 downto 0);
		dinb: IN std_logic_VECTOR(15 downto 0);
		doutb: OUT std_logic_VECTOR(15 downto 0));
	end component;




-- --------ARCHITECTURE BODY BEGINS -----------------------------------------------
begin




	-- COMPONENT INSTALLATIONS (connecting the IPs to local signals) ---------------
	inst_dsp_instr_buffer : dsp_instr_buffer
		port map (
			clka => clka,
			ena => ena,
			wea => wea,
			addra => addra,
			dina => dina,
			douta => douta,
			clkb => clkb,
			enb => enb,
			web => web,
			addrb => addrb,
			dinb => dinb,
			doutb => doutb);	


	-- MAIN LOGIC: -----------------------------------------------------------------

	dsp_refclk <= x25m_clkin;
	dsp_reset_n <= not (dspif_reset or dsp_reset_control_frompcie);
	
	--boot mode = 0010 = FIFO boot
	dsp_bmode(3) <= '0';
	dsp_bmode(2) <= '0';
	dsp_bmode(1) <= '1';
	dsp_bmode(0) <= '0';

	--interrupts:
	dsp_interrupt  <= dsp_irq_frompcie;
	dsp_nmi_n  <= not dsp_nmi_frompcie;
	
	--uart loopback for test:
	dsp_uart_tx <= dsp_uart_rx;
	
	dsp_flag_to_dsp <= dsp_flag_from_pcie;
	dsp_flag_to_pcie <= dsp_flag_from_dsp;


	--HOST INTERFACE: --------------------------------------
	--not used at the moment:
	dsp_hostif_data <= "ZZZZZZZZ";
	dsp_hostif_ce_n <= '1';
	dsp_hostif_rd_n <= '1';
	dsp_hostif_wr_n <= '1';
	dsp_hostif_addr <= "01"; 
	--?? <= dsp_hostif_ack 


	--ASYNCHRONOUS SLAVE BUS INTERFACE:----------------------
	dsp_dmar1_n <='1';
	 process (dspif_reset, dsp_reset_control_frompcie, dsp_ioclk, dsp_ams_n, dsp_aoe_n,
				dsp_fifo_read_data, dsp_fifo_address_counter, async_read_data, dsp_awe_n,
				dsp_read_data_ready, dsp_fifo_address_counter, dsp_aoe_n_previous)
    begin
       if (dspif_reset='1' or dsp_reset_control_frompcie='1') then
           dsp_data <= "ZZZZZZZZZZZZZZZZ";
			  dsp_fifo_address_counter <= "0000000000000000";
			  async_write_data <= "0000000000000000";
			  dsp_ardy <= '1';
			  dsp_writes <= '0';
			  dsp_reads <= '0';
			  dsp_aoe_n_previous <= dsp_aoe_n;
       elsif (dsp_ioclk'event and dsp_ioclk='1') then
		 
         dsp_aoe_n_previous <= dsp_aoe_n;
			
			--boot from FIFO: AMS3_n=0--------
			if (dsp_ams_n(3)='0') then
			  --read
			  if (dsp_aoe_n='0' and dsp_hwait='0') then
			    dsp_data <= dsp_fifo_read_data;
				 dsp_ardy <= '1';
			  elsif (dsp_aoe_n='0' and dsp_hwait='1') then
			    dsp_data <= dsp_fifo_read_data;
				 dsp_ardy <= '1';			  
			  elsif (dsp_aoe_n_previous='0' and dsp_aoe_n='1') then
			    dsp_fifo_address_counter <= dsp_fifo_address_counter +1;
			  --idle
			  else
			    dsp_data <= "ZZZZZZZZZZZZZZZZ";
				 dsp_ardy <= '1';
			  end if;
			  
			--access to data memory: AMS1_n=0-------
			elsif (dsp_ams_n(1)='0') then
			  dsp_address_latched(25 downto 1) <= dsp_addr(25 downto 1);
			  dsp_address_latched(0) <= not dsp_abe_n(0);
			  --read
			  if (dsp_aoe_n='0') then
			    dsp_data <= async_read_data;
				 dsp_ardy <= dsp_read_data_ready;
				 dsp_writes <= '0';
			    dsp_reads <= '1';
			  --write
			  elsif (dsp_awe_n='0') then
			    dsp_data <= "ZZZZZZZZZZZZZZZZ";
				 async_write_data <= dsp_data;
				 dsp_ardy <= '1';
				 dsp_writes <= '1';
			    dsp_reads <= '0';		 
			  --idle
			  else
			    dsp_data <= "ZZZZZZZZZZZZZZZZ";
				 dsp_ardy <= '1';
				 dsp_writes <= '0';
			    dsp_reads <= '0';
			  end if;
			  
			--no access: -------
			else
			  dsp_data <= "ZZZZZZZZZZZZZZZZ";
			  dsp_ardy <= '1';
			  dsp_writes <= '0';
			  dsp_reads <= '0';
			end if;
       end if;
    end process;




	--ACCESS THE INSTRUCTION BUFFER BY THE DSP:-------------------
	clkb <= dsp_ioclk;
	enb <= not dsp_ams_n(3);
	web <= "0";
	addrb <= dsp_fifo_address_counter(14 downto 0);
	dinb <= "0000000000000000";
	dsp_fifo_read_data <= doutb;
	



	--ACCESS THE INSTRUCTION BUFFER BY THE PCIE:------------------
	clka <= dsp_instr_wb_clk_o;
	ena <= dsp_instr_wb_stb_o;
	wea <= wea_vectorize;
	addra <= dsp_instr_wb_addr_o(13 downto 0);
	dina <= dsp_instr_wb_data_o;
	dsp_instr_wb_data_i <= douta;
	dsp_instr_wb_ack_i <= dsp_instr_wb_stb_o;

	wea_vectorize(0) <= dsp_instr_wb_stb_o and dsp_instr_wb_wr_o;



	--ACCESS THE DDR MEMORY THROUGH WISHBONE BUS:-----------------------	
	--Also do access packing between the 16bit DSP bus and the on-chip 32bit wishbone bus.
	--This is a fast wishbone master, supporting very long wait states through ack.
	
	dspif_wb_clk_o <= dsp_ioclk;
	--wishbone address (byte address, but dword aligned only):
	dspif_wb_addr_o(25 downto 2) <= dsp_address_latched(25 downto 2);
	dspif_wb_addr_o(1 downto 0) <= "00"; --only dword aligned accesses will happen
	dsp_upperlower <= dsp_address_latched(1); --0=lower word
	dspif_wb_sel_o (3 downto 2) <= dsp_upperlower_vectorize and (not dsp_abe_n (1 downto 0));
	dspif_wb_sel_o (1 downto 0) <= (not dsp_upperlower_vectorize) and (not dsp_abe_n (1 downto 0));	
	
	dsp_upperlower_vectorize(0) <= dsp_upperlower;
	dsp_upperlower_vectorize(1) <= dsp_upperlower;
	
    process (dspif_reset, dsp_ioclk, wb_state, async_write_data, dsp_reads, dsp_writes, dsp_upperlower,
				dspif_wb_ack_i, dspif_wb_data_i) 
    begin
	 if (dspif_reset ='1') then 
       wb_state <= "000";
		 dspif_wb_data_o <= "00000000000000000000000000000000";
		 dspif_wb_stb_o <=  '0';
		 dspif_wb_cyc_o <=  '0';
		 dspif_wb_wr_o <=  '0';		 
		 dsp_read_data_ready <='0';
    else
      if (dsp_ioclk'event and dsp_ioclk = '1') then
		case ( wb_state ) is

         --********** IDLE STATE. waiting  **********
			when "000" =>   --0        
                if (dsp_reads='1') then
                  wb_state <= "001"; --1
						 dspif_wb_data_o <= "00000000000000000000000000000000";
						 dspif_wb_stb_o <=  '0';
						 dspif_wb_cyc_o <=  '0';
						 dspif_wb_wr_o <=  '0';	
						 dsp_read_data_ready <='0';
                elsif (dsp_writes='1') then
                  wb_state <= "010"; --2		
						if (dsp_upperlower='1') then --0=lower word
						  dspif_wb_data_o(31 downto 16) <= async_write_data(15 downto 0);
						  dspif_wb_data_o(15 downto 0) <= "0000000000000000";
						else
						  dspif_wb_data_o(31 downto 16) <= "0000000000000000";
						  dspif_wb_data_o(15 downto 0) <= async_write_data(15 downto 0);
						end if;						 
						dspif_wb_stb_o <=  '1';
						dspif_wb_cyc_o <=  '1';
						dspif_wb_wr_o <=  '1';	
						dsp_read_data_ready <='0';				
					 else --stay in idle
						 dspif_wb_data_o <= "00000000000000000000000000000000";
						 dspif_wb_stb_o <=  '0';
						 dspif_wb_cyc_o <=  '0';
						 dspif_wb_wr_o <=  '0';		
						 dsp_read_data_ready <='0';
					 end if;    

         --********** WRITE CYCLE  **********
			when "010" =>   --2        
					if (dspif_wb_ack_i='1') then
					  wb_state <= "000"; --0, idle	
					end if;
					dspif_wb_stb_o <=  '1';
					dspif_wb_cyc_o <=  '1';
					dspif_wb_wr_o <=  '1';	
					dsp_read_data_ready <='0';

         --********** READ CYCLE  **********
			when "001" =>   --1        
					if (dspif_wb_ack_i='1') then
					  wb_state <= "000"; --0, idle	
					  dsp_read_data_ready <='1';
					end if;
					if (dsp_upperlower='1') then --0=lower word
						  async_read_data(15 downto 0) <= dspif_wb_data_i(31 downto 16);
					else
						  async_read_data(15 downto 0) <= dspif_wb_data_i(15 downto 0);
					end if;						

			when others => --error
               wb_state <= "000"; --0

		end case;     
       end if;        
    end if;
    end process;







-- -------- END OF FILE -----------------------------------------------------------
end Behavioral;
