--*****************************************************************************
-- (c) Copyright 2009 Xilinx, Inc. All rights reserved.
--
-- This file contains confidential and proprietary information
-- of Xilinx, Inc. and is protected under U.S. and
-- international copyright and other intellectual property
-- laws.
--
-- DISCLAIMER
-- This disclaimer is not a license and does not grant any
-- rights to the materials distributed herewith. Except as
-- otherwise provided in a valid license issued to you by
-- Xilinx, and to the maximum extent permitted by applicable
-- law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
-- WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
-- AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
-- BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
-- INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
-- (2) Xilinx shall not be liable (whether in contract or tort,
-- including negligence, or under any other theory of
-- liability) for any loss or damage of any kind or nature
-- related to, arising under or in connection with these
-- materials, including for any direct, or any indirect,
-- special, incidental, or consequential loss or damage
-- (including loss of data, profits, goodwill, or any type of
-- loss or damage suffered as a result of any action brought
-- by a third party) even if such damage or loss was
-- reasonably foreseeable or Xilinx had been advised of the
-- possibility of the same.
--
-- CRITICAL APPLICATIONS
-- Xilinx products are not designed or intended to be fail-
-- safe, or for use in any application requiring fail-safe
-- performance, such as life-support or safety devices or
-- systems, Class III medical devices, nuclear facilities,
-- applications related to the deployment of airbags, or any
-- other applications that could lead to death, personal
-- injury, or severe property or environmental damage
-- (individually and collectively, "Critical
-- Applications"). Customer assumes the sole risk and
-- liability of any use of Xilinx products in Critical
-- Applications, subject only to applicable laws and
-- regulations governing limitations on product liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
-- PART OF THIS FILE AT ALL TIMES.
--
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: %version
--  \   \         Application: MIG
--  /   /         Filename: pipeline_inserter.vhd
-- /___/   /\     Date Last Modified: $Date: 2009/08/17 16:09:55 $
-- \   \  /  \    Date Created: Jul 03 2009
--  \___\/\___\
--
-- Device: Spartan6
-- Design Name: DDR/DDR2/DDR3/LPDDR 
-- Purpose: Insert pipelines.
-- Reference:
-- Revision History:

--*****************************************************************************


LIBRARY ieee;
   USE ieee.std_logic_1164.all;
   USE ieee.std_logic_unsigned.all;
   USE ieee.numeric_std.all;


ENTITY pipeline_inserter IS
   GENERIC (
      DATA_WIDTH            : INTEGER := 32;
      PIPE_STAGES           : INTEGER := 1
   );
   PORT (      
      data_i                : IN STD_LOGIC_VECTOR(DATA_WIDTH-1 DOWNTO 0);
      clk_i                 : IN STD_LOGIC;
      en_i                  : IN STD_LOGIC;
      
      data_o                : OUT STD_LOGIC_VECTOR(DATA_WIDTH-1 DOWNTO 0)
   );
END pipeline_inserter;

ARCHITECTURE pipeline OF pipeline_inserter IS

   TYPE pipe_arr IS ARRAY (PIPE_STAGES+1 DOWNTO 0) OF STD_LOGIC_VECTOR(DATA_WIDTH - 1 DOWNTO 0);
   SIGNAL pipe_array                : pipe_arr;

 BEGIN

   --**********************************************************
   --  No Pipeline
   --**********************************************************

   no_pipe: if (PIPE_STAGES = 0) GENERATE 

      data_o <= data_i;

   END GENERATE;


  --**********************************************************
  -- Add Pipeline
  --********************************************************** 

  add_pipe: if(PIPE_STAGES > 0) GENERATE
   
    PROCESS BEGIN  
      pipe_init: for j in 0 to PIPE_STAGES - 1    loop
        pipe_array(j) <= std_logic_vector(to_unsigned(0, DATA_WIDTH));
      END LOOP;  
      WAIT;
    END PROCESS;

    data_o <= pipe_array(PIPE_STAGES-1);
   
 
    PROCESS (clk_i)
      BEGIN
      IF (clk_i'EVENT AND clk_i = '1') THEN
        IF (en_i = '1') THEN
          pipe_array(0) <= data_i;     
        END IF;
      END IF;
    END PROCESS;


    pipe_stage: FOR i IN 1 TO  (PIPE_STAGES - 1) GENERATE   

      PROCESS (clk_i)
      BEGIN
        IF (clk_i'EVENT AND clk_i = '1') THEN
          IF (en_i = '1') THEN
            pipe_array(i) <= pipe_array(i-1);
          END IF;
        END IF;
      END PROCESS;
    END GENERATE;

  END GENERATE; 

END pipeline;
