----------------------------------------------------------------------------------
-- Company: Home, hobby project
-- Engineer: Istvan Nagy
-- 
-- Create Date:    15:40:36 05/30/2010 
-- Design Name:  Spartan6-Blackfin-Board
-- Module Name:    s6bf_board_top - Behavioral 
-- Project Name: 
-- Target Devices: XC6SLX45T
-- Tool versions: ISE 11.4
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
--  ===========SYSTEM:=============
--
--  DDR2_chip (64Mbytes,500MT/s,x16)
--    |
-- [MEM.CO.]
--  | | | |_____ADC_INTERFACE<--ADS5517 A/D Converter (200MSPS,LVDS)
--  | | |_______DSP_INTERFACE<-->ADSP-BF544 DSP, (500MHz)
--  | |_________AURORA_INTERFACE<-->RJ45 connector, (AC coupled, 2lanes, 2.5Gbps)
--  |___________PCIE-ENDPOINT,BAR1<-->card_edge (x1, PCIe 1.1)
--
-- PCIE endpoint BAR0 usage:
--  address 0...64kB: Control and status registers for the other interfaces.
--  address 64kB...128kB: DSP firmware FIFO, 64KB storage.
--
-- ADDRESSES ARE BYTE ADDRESSES. so the dword peripherals must ignore addr[1:0] 
--
-- Control&Status Registers BAR0 Address Map (0000-1FFFF: LSB-bytes only):
-- -------------------------------------
--  addr   name            bit7   bit6   bit5   bit4   bit3   bit2   bit1   bit0
--  00h    PGA_gain_reg    -      -      b5     b4     b3     b2     b1     b0
--  04h    ADC_cont_reg    start  finish oversh -      -      -      -      -
--  08h    ADC_trigg_reg   b7     b6     b5     b4     b3     b2     b1     b0
--  0Ah    DSP_contr_reg   boot   reset  irq    nmi    -      -      -      -
--  10h    DSP_flag_reg    -      -      -      -      out1   out0   in1    in0
--  100h to 17F: Aurora config/status registers
--
-- 10000
--  to     DSP_instr_FIFO
-- 1FFFF
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity s6bf_board_top is
    
	 Port ( --TOPLEVEL PORTS-----------------------
			 --ADC INTERFACE---
			  ADC_CLK_REFOUT_P : out    std_logic; --set LVDS+Rs --200MHz
			  ADC_CLK_REFOUT_N  : out    std_logic; --set LVDS+Rs			 
			  ADC_CLK_IN_P   : in    std_logic; --set LVDS+Rp, use GCLK-pin
			  ADC_CLK_IN_N   : in    std_logic; --set LVDS+Rp, use GCLK-pin
			  ADC_DATA_IN_P   : in    std_logic_vector(5 downto 0); --set LVDS+Rp, same bufio2-region
			  ADC_DATA_IN_N   : in    std_logic_vector(5 downto 0); --set LVDS+Rp, same bufio2-region
			  ADC_OE : out std_logic;
			  ADC_OVR : in std_logic;
			  ADC_SDATA : out std_logic;
			  ADC_SCLK : out std_logic;
			  ADC_SEN : out std_logic;
			  ADC_RESET : out std_logic;
			  PGA_DATA_out  : out    std_logic_vector(5 downto 0); 
			  PGA_STROBE : out std_logic; 
			  --DSP INTERFACE:
			  dsp_reset_n : out  STD_LOGIC;
			  dsp_refclk :  : out  STD_LOGIC;	
			  dsp_ioclk: in  STD_LOGIC;  --to a GCLK pin!
			  dsp_bmode : out  STD_LOGIC_VECTOR (3 downto 0);
			  dsp_dmar1_n : out  STD_LOGIC;
			  dsp_hwait : in  STD_LOGIC;
			  dsp_ams_n : in  STD_LOGIC_VECTOR (3 downto 0);
			  dsp_data : inout  STD_LOGIC_VECTOR (15 downto 0);
			  dsp_abe_n :in  STD_LOGIC_VECTOR (1 downto 0);
			  dsp_awe_n : in  STD_LOGIC;  --to a GCLK pin!
			  dsp_aoe_n : in  STD_LOGIC;
			  dsp_are_n : in  STD_LOGIC;  --to a GCLK pin!
			  dsp_ardy : out  STD_LOGIC;
			  dsp_addr : in STD_LOGIC_VECTOR (25 downto 0);
			  dsp_flag_from_dsp : in  STD_LOGIC_VECTOR (1 downto 0);
			  dsp_flag_to_dsp : out  STD_LOGIC_VECTOR (1 downto 0);
			  dsp_uart_tx : out  STD_LOGIC;
			  dsp_uart_rx : in  STD_LOGIC;
			  dsp_interrupt : out  STD_LOGIC;
			  dsp_nmi : out  STD_LOGIC;
			  dsp_hostif_data :inout  STD_LOGIC_VECTOR (7 downto 0);
			  dsp_hostif_ce_n : out  STD_LOGIC;
			  dsp_hostif_rd_n : out  STD_LOGIC;
			  dsp_hostif_wr_n : out  STD_LOGIC;
			  dsp_hostif_addr : out  STD_LOGIC_VECTOR (1 downto 0);
			  dsp_hostif_ack : in  STD_LOGIC;
			  --AURORA INTERFACE:
           aurora_RXP  : in std_logic_vector(0 to 1);
           aurora_RXN  : in std_logic_vector(0 to 1);
           aurora_TXP  : out std_logic_vector(0 to 1);
           aurora_TXN  : out std_logic_vector(0 to 1);
			  --DDR MEMORY PINS:
			  mcb3_dram_dq        : inout  std_logic_vector(15 downto 0);
			  mcb3_dram_a         : out std_logic_vector(12 downto 0);
			  mcb3_dram_ba        : out std_logic_vector(1 downto 0);
			  mcb3_dram_ras_n     : out std_logic;
			  mcb3_dram_cas_n     : out std_logic;
			  mcb3_dram_we_n      : out std_logic;
			  mcb3_dram_odt       : out std_logic;
			  mcb3_dram_cke       : out std_logic;
			  mcb3_dram_dm        : out std_logic;
			  mcb3_dram_udqs      : inout  std_logic;
			  mcb3_dram_udqs_n    : inout  std_logic;
			  mcb3_rzq            : inout  std_logic;
			  mcb3_zio            : inout  std_logic;
			  mcb3_dram_udm       : out std_logic;
			  mcb3_dram_dqs       : inout  std_logic;
			  mcb3_dram_dqs_n     : inout  std_logic;
			  mcb3_dram_ck        : out std_logic;
			  mcb3_dram_ck_n      : out std_logic;		
			  --PCI-EXPRESS INTERFACE:
			  pci_exp_txp             : out std_logic;
			  pci_exp_txn             : out std_logic;
			  pci_exp_rxp             : in  std_logic;
			  pci_exp_rxn             : in  std_logic;	 
			  sys_clk_n                 : in  std_logic;
			  sys_clk_p                 : in  std_logic;
			  sys_reset_n             : in  std_logic;					  
			  -- OTHER:
			  x25m_clkin in  STD_LOGIC;
			  led0 : out  STD_LOGIC;
			  led1 : out  STD_LOGIC;
			  led2 : out  STD_LOGIC;
			  led3 : out  STD_LOGIC
			  );
end s6bf_board_top;

-- --------ARCHITECTURE BEGINS ----------------------------------------------------
architecture Behavioral of s6bf_board_top is

   -- Internal Signals ------------------------------------------------------------
	--SIGNAL dummy : std_logic_vector(15 downto 0);	--write data bus
	
	--ADC controller onchip signals:
	SIGNAL adcif_wb_data_o :   std_logic_vector(31 downto 0); 
	SIGNAL adcif_wb_data_i :   std_logic_vector(31 downto 0);
	SIGNAL adcif_wb_addr_o :   std_logic_vector(25 downto 0);
	SIGNAL adcif_wb_cyc_o :   std_logic;
	SIGNAL adcif_wb_stb_o :   std_logic;
	SIGNAL adcif_wb_wr_o :   std_logic;
	SIGNAL adcif_wb_ack_i :   std_logic;
	SIGNAL adcif_wb_clk_o :   std_logic; --50MHz
	SIGNAL --CONTROL:
	SIGNAL adc_overshoot_detected :   std_logic;
	SIGNAL pga_ga  :   std_logic_vector(5 downto 0); 
	SIGNAL adc_triggerlevel :   std_logic_vector(7 downto 0); 
	SIGNAL adc_start :    std_logic; --edge sensitive
	SIGNAL adc_f ished :    std_logic;
	SIGNAL --SYSTEM:
	SIGNAL x25m_clk  :   std_logic;
	SIGNAL adcif_reset :   std_logic --active high

	--DSP INTERFACE onchip signals
	--DATA BUS (wishbone):
	SIGNAL dspif_wb_data_o :  std_logic_vector(31 downto 0); 
	SIGNAL dspif_wb_data_i :  std_logic_vector(31 downto 0);
	SIGNAL dspif_wb_addr_o :  std_logic_vector(25 downto 0);
	SIGNAL dspif_wb_cyc_o :  std_logic;
	SIGNAL dspif_wb_stb_o :  std_logic;
	SIGNAL dspif_wb_wr_o :  std_logic;
	SIGNAL dspif_wb_ack_i :  std_logic;
	SIGNAL dspif_wb_clk_o :  std_logic; 
	--instruction buffer :
	SIGNAL dsp_instr_wb_data_o :  std_logic_vector(31 downto 0); 
	SIGNAL dsp_instr_wb_data_i :  std_logic_vector(31 downto 0);
	SIGNAL dsp_instr_wb_addr_o :  std_logic_vector(15 downto 0);
	SIGNAL dsp_instr_wb_cyc_o :  std_logic;
	SIGNAL dsp_instr_wb_stb_o :  std_logic;
	SIGNAL dsp_instr_wb_wr_o :  std_logic;
	SIGNAL dsp_instr_wb_ack_i :  std_logic;
	SIGNAL dsp_instr_wb_clk_o :  std_logic; 
	--Control:
	SIGNAL dsp_start_boot_frompcie :  std_logic;
	SIGNAL dsp_reset_control_frompcie :  std_logic;
	SIGNAL dsp_irq_frompcie :  std_logic; 
	SIGNAL dsp_nmi_frompcie :  std_logic; 
	SIGNAL dsp_flag_to_pcie :   STD_LOGIC_VECTOR (1 downto 0);
	SIGNAL dsp_flag_from_pcie :   STD_LOGIC_VECTOR (1 downto 0);	SIGNAL 
	--System:
	SIGNAL x25m_clkin :  std_logic;
	SIGNAL dspif_reset :  std_logic --active high

	--AURORA INTERFACE ONCHIP SIGNALS:
	--DATA BUS (wishbone):
	SIGNAL auroraif_wb_data_o : std_logic_vector(31 downto 0); 
	SIGNAL auroraif_wb_data_i : std_logic_vector(31 downto 0);
	SIGNAL auroraif_wb_addr_o : std_logic_vector(25 downto 0);
	SIGNAL auroraif_wb_cyc_o : std_logic;
	SIGNAL auroraif_wb_stb_o : std_logic;
	SIGNAL auroraif_wb_wr_o : std_logic;
	SIGNAL auroraif_wb_ack_i : std_logic;
	SIGNAL auroraif_wb_clk_o : std_logic; 
	--instruction registers :
	SIGNAL aurora_config_wb_data_o : std_logic_vector(31 downto 0); 
	SIGNAL aurora_config_wb_data_i : std_logic_vector(31 downto 0);
	SIGNAL aurora_config_wb_addr_i : std_logic_vector(4 downto 0);
	SIGNAL aurora_config_wb_cyc_i : std_logic;
	SIGNAL aurora_config_wb_stb_i : std_logic;
	SIGNAL aurora_config_wb_wr_i : std_logic;
	SIGNAL aurora_config_wb_ack_o : std_logic;
	SIGNAL aurora_config_wb_clk_i : std_logic; 		
	--completion flags	SIGNAL 
	SIGNAL aurora_reset : std_logic;
	SIGNAL x25m_clk : std_logic;
	SIGNAL tx_completed :  STD_LOGIC;
	SIGNAL rx_completed :  STD_LOGIC;

	--MEMORY INTERFACE ONCHIP SIGNALS:
	--channel-0
	SIGNAL channel0_wb_data_o : std_logic_vector(31 downto 0); 
	SIGNAL channel0_wb_data_i : std_logic_vector(31 downto 0);
	SIGNAL channel0_wb_addr_i : std_logic_vector(25 downto 0);
	SIGNAL channel0_wb_cyc_i : std_logic;
	SIGNAL channel0_wb_stb_i : std_logic;
	SIGNAL channel0_wb_wr_i : std_logic;
	SIGNAL channel0_wb_ack_o : std_logic;
	SIGNAL channel0_wb_clk_i : std_logic; 	
	--channel-1
	SIGNAL channel1_wb_data_o : std_logic_vector(31 downto 0); 
	SIGNAL channel1_wb_data_i : std_logic_vector(31 downto 0);
	SIGNAL channel1_wb_addr_i : std_logic_vector(25 downto 0);
	SIGNAL channel1_wb_cyc_i : std_logic;
	SIGNAL channel1_wb_stb_i : std_logic;
	SIGNAL channel1_wb_wr_i : std_logic;
	SIGNAL channel1_wb_ack_o : std_logic;
	SIGNAL channel1_wb_clk_i : std_logic; 
	--channel-2
	SIGNAL channel2_wb_data_o : std_logic_vector(31 downto 0); 
	SIGNAL channel2_wb_data_i : std_logic_vector(31 downto 0);
	SIGNAL channel2_wb_addr_i : std_logic_vector(25 downto 0);
	SIGNAL channel2_wb_cyc_i : std_logic;
	SIGNAL channel2_wb_stb_i : std_logic;
	SIGNAL channel2_wb_wr_i : std_logic;
	SIGNAL channel2_wb_ack_o : std_logic;
	SIGNAL channel2_wb_clk_i : std_logic; 
	--channel-3
	SIGNAL channel3_wb_data_o : std_logic_vector(31 downto 0); 
	SIGNAL channel3_wb_data_i : std_logic_vector(31 downto 0);
	SIGNAL channel3_wb_addr_i : std_logic_vector(25 downto 0);
	SIGNAL channel3_wb_cyc_i : std_logic;
	SIGNAL channel3_wb_stb_i : std_logic;
	SIGNAL channel3_wb_wr_i : std_logic;
	SIGNAL channel3_wb_ack_o : std_logic;
	SIGNAL channel3_wb_clk_i : std_logic; 

	--PCIE-INTERFACE ON CHIP PORTS:
	--DATA BUS (wishbone):
	SIGNAL pcieif_wb_data_o : std_logic_vector(31 downto 0); 
	SIGNAL pcieif_wb_data_i : std_logic_vector(31 downto 0);
	SIGNAL pcieif_wb_addr_o : std_logic_vector(25 downto 0);
	SIGNAL pcieif_wb_cyc_o : std_logic;
	SIGNAL pcieif_wb_stb_o : std_logic;
	SIGNAL pcieif_wb_wr_o : std_logic;
	SIGNAL pcieif_wb_ack_i : std_logic;
	SIGNAL pcieif_wb_clk_o : std_logic; --62.5MHz			
	--OTHER:
	SIGNAL pcie_resetout  :  std_logic;


	-- COMPONENT DECLARATIONS (introducing the IPs) --------------------------------

	COMPONENT s6bfip_adc
	PORT(
		ADC_CLK_IN_P : IN std_logic;
		ADC_CLK_IN_N : IN std_logic;
		ADC_DATA_IN_P : IN std_logic_vector(5 downto 0);
		ADC_DATA_IN_N : IN std_logic_vector(5 downto 0);
		ADC_OVR : IN std_logic;
		adcif_wb_data_i : IN std_logic_vector(31 downto 0);
		adcif_wb_ack_i : IN std_logic;
		pga_gain : IN std_logic_vector(5 downto 0);
		adc_triggerlevel : IN std_logic_vector(7 downto 0);
		adc_start : IN std_logic;
		x25m_clkin : IN std_logic;
		adcif_reset : IN std_logic;          
		ADC_CLK_REFOUT_P : OUT std_logic;
		ADC_CLK_REFOUT_N : OUT std_logic;
		ADC_OE : OUT std_logic;
		ADC_SDATA : OUT std_logic;
		ADC_SCLK : OUT std_logic;
		ADC_SEN : OUT std_logic;
		ADC_RESET : OUT std_logic;
		PGA_DATA_out : OUT std_logic_vector(5 downto 0);
		PGA_STROBE : OUT std_logic;
		adcif_wb_data_o : OUT std_logic_vector(31 downto 0);
		adcif_wb_addr_o : OUT std_logic_vector(25 downto 0);
		adcif_wb_cyc_o : OUT std_logic;
		adcif_wb_stb_o : OUT std_logic;
		adcif_wb_wr_o : OUT std_logic;
		adcif_wb_clk_o : OUT std_logic;
		adc_overshoot_detected : OUT std_logic;
		adc_finished : OUT std_logic
		);
	END COMPONENT;



	COMPONENT s6bfip_dsp
	PORT(
		dsp_ioclk : IN std_logic;
		dsp_hwait : IN std_logic;
		dsp_ams_n : IN std_logic_vector(3 downto 0);
		dsp_abe_n : IN std_logic_vector(1 downto 0);
		dsp_awe_n : IN std_logic;
		dsp_aoe_n : IN std_logic;
		dsp_are_n : IN std_logic;
		dsp_addr : IN std_logic_vector(25 downto 0);
		dsp_flag_from_dsp : IN std_logic_vector(1 downto 0);
		dsp_uart_rx : IN std_logic;
		dsp_hostif_ack : IN std_logic;
		dspif_wb_data_i : IN std_logic_vector(31 downto 0);
		dspif_wb_ack_i : IN std_logic;
		dsp_instr_wb_data_o : IN std_logic_vector(31 downto 0);
		dsp_instr_wb_addr_o : IN std_logic_vector(15 downto 0);
		dsp_instr_wb_cyc_o : IN std_logic;
		dsp_instr_wb_stb_o : IN std_logic;
		dsp_instr_wb_wr_o : IN std_logic;
		dsp_instr_wb_clk_o : IN std_logic;
		dsp_start_boot_frompcie : IN std_logic;
		dsp_reset_control_frompcie : IN std_logic;
		dsp_irq_frompcie : IN std_logic;
		dsp_nmi_frompcie : IN std_logic;
		dsp_flag_from_pcie : IN std_logic_vector(1 downto 0);
		x25m_clkin : IN std_logic;
		dspif_reset : IN std_logic;    
		dsp_data : INOUT std_logic_vector(15 downto 0);
		dsp_hostif_data : INOUT std_logic_vector(7 downto 0);      
		dsp_reset_n : OUT std_logic;
		dsp_refclk : OUT std_logic;
		dsp_bmode : OUT std_logic_vector(3 downto 0);
		dsp_dmar1_n : OUT std_logic;
		dsp_ardy : OUT std_logic;
		dsp_flag_to_dsp : OUT std_logic_vector(1 downto 0);
		dsp_uart_tx : OUT std_logic;
		dsp_interrupt : OUT std_logic;
		dsp_nmi : OUT std_logic;
		dsp_hostif_ce_n : OUT std_logic;
		dsp_hostif_rd_n : OUT std_logic;
		dsp_hostif_wr_n : OUT std_logic;
		dsp_hostif_addr : OUT std_logic_vector(1 downto 0);
		dspif_wb_data_o : OUT std_logic_vector(31 downto 0);
		dspif_wb_addr_o : OUT std_logic_vector(25 downto 0);
		dspif_wb_cyc_o : OUT std_logic;
		dspif_wb_stb_o : OUT std_logic;
		dspif_wb_wr_o : OUT std_logic;
		dspif_wb_clk_o : OUT std_logic;
		dsp_instr_wb_data_i : OUT std_logic_vector(31 downto 0);
		dsp_instr_wb_ack_i : OUT std_logic;
		dsp_flag_to_pcie : OUT std_logic_vector(1 downto 0)
		);
	END COMPONENT;

	COMPONENT s6bfip_aurora
    Port ( --FPGA PINS (EXTERNAL):
           aurora_RXP  : in std_logic_vector(0 to 1);
           aurora_RXN  : in std_logic_vector(0 to 1);
           aurora_TXP  : out std_logic_vector(0 to 1);
           aurora_TXN  : out std_logic_vector(0 to 1);
			  --INTERNAL PORTS:
			  --DATA BUS (wishbone):
			  auroraif_wb_data_o : out std_logic_vector(31 downto 0); 
			  auroraif_wb_data_i : in std_logic_vector(31 downto 0);
			  auroraif_wb_addr_o : out std_logic_vector(25 downto 0);
			  auroraif_wb_cyc_o : out std_logic;
			  auroraif_wb_stb_o : out std_logic;
			  auroraif_wb_wr_o : out std_logic;
			  auroraif_wb_ack_i : in std_logic;
			  auroraif_wb_clk_o : out std_logic; 
			  --instruction registers :
			  aurora_config_wb_data_o : in std_logic_vector(31 downto 0); 
			  aurora_config_wb_data_i : out std_logic_vector(31 downto 0);
			  aurora_config_wb_addr_i : in std_logic_vector(4 downto 0);
			  aurora_config_wb_cyc_i : in std_logic;
			  aurora_config_wb_stb_i : in std_logic;
			  aurora_config_wb_wr_i : in std_logic;
			  aurora_config_wb_ack_o : out std_logic;
			  aurora_config_wb_clk_i : in std_logic; 		
			  --completion flags			  
			  aurora_reset : in std_logic;
			  x25m_clk : in std_logic;
			  tx_completed : out  STD_LOGIC;
			  rx_completed : out  STD_LOGIC
			 );
	END COMPONENT;


	COMPONENT s6bfip_memory
    Port (--FPGA_PINS (EXTERNAL):
			  mcb3_dram_dq        : inout  std_logic_vector(15 downto 0);
			  mcb3_dram_a         : out std_logic_vector(12 downto 0);
			  mcb3_dram_ba        : out std_logic_vector(1 downto 0);
			  mcb3_dram_ras_n     : out std_logic;
			  mcb3_dram_cas_n     : out std_logic;
			  mcb3_dram_we_n      : out std_logic;
			  mcb3_dram_odt       : out std_logic;
			  mcb3_dram_cke       : out std_logic;
			  mcb3_dram_dm        : out std_logic;
			  mcb3_dram_udqs      : inout  std_logic;
			  mcb3_dram_udqs_n    : inout  std_logic;
			  mcb3_rzq            : inout  std_logic;
			  mcb3_zio            : inout  std_logic;
			  mcb3_dram_udm       : out std_logic;
			  mcb3_dram_dqs       : inout  std_logic;
			  mcb3_dram_dqs_n     : inout  std_logic;
			  mcb3_dram_ck        : out std_logic;
			  mcb3_dram_ck_n      : out std_logic;	
			  --ONCHIP PORTS:
			  --channel-0
			  channel0_wb_data_o : in std_logic_vector(31 downto 0); 
			  channel0_wb_data_i : out std_logic_vector(31 downto 0);
			  channel0_wb_addr_i : in std_logic_vector(25 downto 0);
			  channel0_wb_cyc_i : in std_logic;
			  channel0_wb_stb_i : in std_logic;
			  channel0_wb_wr_i : in std_logic;
			  channel0_wb_ack_o : out std_logic;
			  channel0_wb_clk_i : in std_logic; 	
			  --channel-1
			  channel1_wb_data_o : in std_logic_vector(31 downto 0); 
			  channel1_wb_data_i : out std_logic_vector(31 downto 0);
			  channel1_wb_addr_i : in std_logic_vector(25 downto 0);
			  channel1_wb_cyc_i : in std_logic;
			  channel1_wb_stb_i : in std_logic;
			  channel1_wb_wr_i : in std_logic;
			  channel1_wb_ack_o : out std_logic;
			  channel1_wb_clk_i : in std_logic; 
			  --channel-2
			  channel2_wb_data_o : in std_logic_vector(31 downto 0); 
			  channel2_wb_data_i : out std_logic_vector(31 downto 0);
			  channel2_wb_addr_i : in std_logic_vector(25 downto 0);
			  channel2_wb_cyc_i : in std_logic;
			  channel2_wb_stb_i : in std_logic;
			  channel2_wb_wr_i : in std_logic;
			  channel2_wb_ack_o : out std_logic;
			  channel2_wb_clk_i : in std_logic; 
			  --channel-3
			  channel3_wb_data_o : in std_logic_vector(31 downto 0); 
			  channel3_wb_data_i : out std_logic_vector(31 downto 0);
			  channel3_wb_addr_i : in std_logic_vector(25 downto 0);
			  channel3_wb_cyc_i : in std_logic;
			  channel3_wb_stb_i : in std_logic;
			  channel3_wb_wr_i : in std_logic;
			  channel3_wb_ack_o : out std_logic;
			  channel3_wb_clk_i : in std_logic; 
			  --System signals:
			  x25m_clk : in std_logic; 
			  reset : in std_logic 
			 );
	END COMPONENT;

	COMPONENT s6bfip_pcie
    Port ( --FPGA PINS(EXTERNAL):
			 pci_exp_txp             : out std_logic;
			 pci_exp_txn             : out std_logic;
			 pci_exp_rxp             : in  std_logic;
			 pci_exp_rxn             : in  std_logic;	 
			 sys_clk_n                 : in  std_logic;
			 sys_clk_p                 : in  std_logic;
			 sys_reset_n             : in  std_logic;			 
			 --ON CHIP PORTS:
			 --DATA BUS (wishbone):
			 pcieif_wb_data_o : out std_logic_vector(31 downto 0); 
			 pcieif_wb_data_i : in std_logic_vector(31 downto 0);
			 pcieif_wb_addr_o : out std_logic_vector(25 downto 0);
			 pcieif_wb_cyc_o : out std_logic;
			 pcieif_wb_stb_o : out std_logic;
			 pcieif_wb_wr_o : out std_logic;
			 pcieif_wb_ack_i : in std_logic;
			 pcieif_wb_clk_o : out std_logic; --62.5MHz			
			 --AURORA control bus:
			  aurora_config_wb_data_o : out std_logic_vector(31 downto 0); 
			  aurora_config_wb_data_i : in std_logic_vector(31 downto 0);
			  aurora_config_wb_addr_i : out std_logic_vector(4 downto 0);
			  aurora_config_wb_cyc_i : out std_logic;
			  aurora_config_wb_stb_i : out std_logic;
			  aurora_config_wb_wr_i : out std_logic;
			  aurora_config_wb_ack_o : in std_logic;
			  aurora_config_wb_clk_i : out std_logic; 
			 --DSP firmware loader bus:
			  dsp_instr_wb_data_o : out std_logic_vector(31 downto 0); 
			  dsp_instr_wb_data_i : in std_logic_vector(31 downto 0);
			  dsp_instr_wb_addr_o : out std_logic_vector(15 downto 0);
			  dsp_instr_wb_cyc_o : out std_logic;
			  dsp_instr_wb_stb_o : out std_logic;
			  dsp_instr_wb_wr_o : out std_logic;
			  dsp_instr_wb_ack_i : in std_logic;
			  dsp_instr_wb_clk_o : out std_logic; 			 
			 --ADC control signals:
			  adc_overshoot_detected : in std_logic;
			  pga_gain : out std_logic_vector(5 downto 0); 
			  adc_triggerlevel : out std_logic_vector(7 downto 0); 
			  adc_start : out  std_logic; --edge sensitive
			  adc_finished : in  std_logic;			 
			 --DSP control signals:
			  dsp_start_boot_frompcie : out std_logic;
			  dsp_reset_control_frompcie : out std_logic;
			  dsp_irq_frompcie : out std_logic; 
			  dsp_nmi_frompcie : out std_logic; 
			  dsp_flag_to_pcie : in  STD_LOGIC_VECTOR (1 downto 0);
			  dsp_flag_from_pcie : out  STD_LOGIC_VECTOR (1 downto 0);				 
			 --OTHER:
			 pcie_resetout  : out std_logic		 
			);
	END COMPONENT;






-- --------ARCHITECTURE BODY BEGINS -----------------------------------------------
begin




	-- COMPONENT INSTALLATIONS (connecting the IPs to local signals) ---------------
	
		
		
		
	--ADC interface			
	Inst_s6bfip_adc: s6bfip_adc PORT MAP(			
		ADC_CLK_REFOUT_P	=>	ADC_CLK_REFOUT_P,
		ADC_CLK_REFOUT_N	=>	ADC_CLK_REFOUT_N,
		ADC_CLK_IN_P	=>	ADC_CLK_IN_P,			
		ADC_CLK_IN_N	=>	ADC_CLK_IN_N,			
		ADC_DATA_IN_P	=>	ADC_DATA_IN_P,			
		ADC_DATA_IN_N	=>	ADC_DATA_IN_N,			
		ADC_OE	=>	ADC_OE,			
		ADC_OVR	=>	ADC_OVR,
		ADC_SDATA	=>	ADC_SDATA,			
		ADC_SCLK	=>	ADC_SCLK,			
		ADC_SEN	=>	ADC_SEN,
		ADC_RESET	=>	ADC_RESET,			
		PGA_DATA_out	=>	PGA_DATA_out,			
		PGA_STROBE	=>	PGA_STROBE,			
		adcif_wb_data_o	=>	adcif_wb_data_o,			
		adcif_wb_data_i	=>	adcif_wb_data_i,			
		adcif_wb_addr_o	=>	adcif_wb_addr_o,			
		adcif_wb_cyc_o	=>	adcif_wb_cyc_o,			
		adcif_wb_stb_o	=>	adcif_wb_stb_o,			
		adcif_wb_wr_o	=>	adcif_wb_wr_o,			
		adcif_wb_ack_i	=>	adcif_wb_ack_i,			
		adcif_wb_clk_o	=>	adcif_wb_clk_o,			
		adc_overshoot_detected	=>	adc_overshoot_detected,
		pga_gain	=>	pga_gain,			
		adc_triggerlevel	=>	adc_triggerlevel,
		adc_start	=>	adc_start,			
		adc_finished	=>	adc_finished,			
		x25m_clkin	=>	x25m_clkin,			
		adcif_reset	=>	adcif_reset				
	);	
		
		
		
	--DSP interface:
	Inst_s6bfip_dsp: s6bfip_dsp PORT MAP(			
		dsp_reset_n	=>	dsp_reset_n,			
		dsp_refclk	=>	dsp_refclk,			
		dsp_ioclk	=>	dsp_ioclk,			
		dsp_bmode	=>	dsp_bmode,			
		dsp_dmar1_n	=>	dsp_dmar1_n,			
		dsp_hwait	=>	dsp_hwait,			
		dsp_ams_n	=>	dsp_ams_n,			
		dsp_data	=>	dsp_data,			
		dsp_abe_n	=>	dsp_abe_n,			
		dsp_awe_n	=>	dsp_awe_n,			
		dsp_aoe_n	=>	dsp_aoe_n,			
		dsp_are_n	=>	dsp_are_n,			
		dsp_ardy	=>	dsp_ardy,			
		dsp_addr	=>	dsp_addr,			
		dsp_flag_from_dsp	=>	dsp_flag_from_dsp,
		dsp_flag_to_dsp	=>	dsp_flag_to_dsp,			
		dsp_uart_tx	=>	dsp_uart_tx,			
		dsp_uart_rx	=>	dsp_uart_rx,			
		dsp_interrupt	=>	dsp_interrupt,			
		dsp_nmi		=>	dsp_nmi,
		dsp_hostif_data	=>	dsp_hostif_data,			
		dsp_hostif_ce_n	=>	dsp_hostif_ce_n,			
		dsp_hostif_rd_n	=>	dsp_hostif_rd_n,			
		dsp_hostif_wr_n	=>	dsp_hostif_wr_n,			
		dsp_hostif_addr	=>	dsp_hostif_addr,			
		dsp_hostif_ack	=>	dsp_hostif_ack,			
		dspif_wb_data_o	=>	dspif_wb_data_o,			
		dspif_wb_data_i	=>	dspif_wb_data_i,			
		dspif_wb_addr_o	=>	dspif_wb_addr_o,			
		dspif_wb_cyc_o	=>	dspif_wb_cyc_o,			
		dspif_wb_stb_o	=>	dspif_wb_stb_o,			
		dspif_wb_wr_o	=>	dspif_wb_wr_o,			
		dspif_wb_ack_i	=>	dspif_wb_ack_i,			
		dspif_wb_clk_o	=>	dspif_wb_clk_o,			
		dsp_instr_wb_data_o	=>	dsp_instr_wb_data_o,
		dsp_instr_wb_data_i	=>	dsp_instr_wb_data_i,
		dsp_instr_wb_addr_o	=>	dsp_instr_wb_addr_o,
		dsp_instr_wb_cyc_o	=>	dsp_instr_wb_cyc_o,
		dsp_instr_wb_stb_o	=>	dsp_instr_wb_stb_o,
		dsp_instr_wb_wr_o	=>	dsp_instr_wb_wr_o,
		dsp_instr_wb_ack_i	=>	dsp_instr_wb_ack_i,
		dsp_instr_wb_clk_o	=>	dsp_instr_wb_clk_o,
		dsp_start_boot_frompcie	=>	dsp_start_boot_frompcie,			
		dsp_reset_control_frompcie	=>	dsp_reset_control_frompcie,			
		dsp_irq_frompcie	=>	dsp_irq_frompcie,
		dsp_nmi_frompcie	=>	dsp_nmi_frompcie,
		dsp_flag_to_pcie	=>	dsp_flag_to_pcie,
		dsp_flag_from_pcie	=>	dsp_flag_from_pcie,
		x25m_clkin	=>	x25m_clkin,			
		dspif_reset	=>	dspif_reset				
	);	
		
		
	Inst_s6bfip_aurora:	s6bfip_aurora	PORT MAP(			
		aurora_RXP	=>	aurora_RXP,		
		aurora_RXN	=>	aurora_RXN,		
		aurora_TXP	=>	aurora_TXP,		
		aurora_TXN	=>	aurora_TXN,		
		--INTERNAL PORTS:		
		--DATA BUS (wishbone):			
		auroraif_wb_data_o	=>	auroraif_wb_data_o,		
		auroraif_wb_data_i	=>	auroraif_wb_data_i,		
		auroraif_wb_addr_o	=>	auroraif_wb_addr_o,		
		auroraif_wb_cyc_o	=>	auroraif_wb_cyc_o,		
		auroraif_wb_stb_o	=>	auroraif_wb_stb_o,		
		auroraif_wb_wr_o	=>	auroraif_wb_wr_o,		
		auroraif_wb_ack_i	=>	auroraif_wb_ack_i,		
		auroraif_wb_clk_o	=>	auroraif_wb_clk_o,		
		--instruction registers	:			
		aurora_config_wb_data_o	=>	aurora_config_wb_data_o,		
		aurora_config_wb_data_i	=>	aurora_config_wb_data_i,		
		aurora_config_wb_addr_o	=>	aurora_config_wb_addr_o,		
		aurora_config_wb_cyc_o	=>	aurora_config_wb_cyc_o,		
		aurora_config_wb_stb_o	=>	aurora_config_wb_stb_o,		
		aurora_config_wb_wr_o	=>	aurora_config_wb_wr_o,		
		aurora_config_wb_ack_i	=>	aurora_config_wb_ack_i,		
		aurora_config_wb_clk_o	=>	aurora_config_wb_clk_o,		
		--completion flags		
		aurora_reset	=>	aurora_reset,		
		x25m_clk	=>	x25m_clk,		
		tx_completed	=>	tx_completed,		
		rx_completed	=>	rx_completed				
	);	
		
		
		
	Inst_s6bfip_memory: s6bfip_memory PORT MAP(			
		mcb3_dram_dq	=>	mcb3_dram_dq,	--vector(15 downto 0)		
		mcb3_dram_a	=>	mcb3_dram_a,    --vector(12 downto 0)				
		mcb3_dram_ba	=>	mcb3_dram_ba,	--vector(1 downto 0)		
		mcb3_dram_ras_n	=>	mcb3_dram_ras_n,		
		mcb3_dram_cas_n	=>	mcb3_dram_cas_n,		
		mcb3_dram_we_n	=>	mcb3_dram_we_n,		
		mcb3_dram_odt	=>	mcb3_dram_odt,		
		mcb3_dram_cke	=>	mcb3_dram_cke,		
		mcb3_dram_dm	=>	mcb3_dram_dm,		
		mcb3_dram_udqs	=>	mcb3_dram_udqs,		
		mcb3_dram_udqs_n	=>	mcb3_dram_udqs_n,		
		mcb3_rzq	=>	mcb3_rzq,		
		mcb3_zio	=>	mcb3_zio,		
		mcb3_dram_udm	=>	mcb3_dram_udm,		
		mcb3_dram_dqs	=>	mcb3_dram_dqs,		
		mcb3_dram_dqs_n	=>	mcb3_dram_dqs_n,		
		mcb3_dram_ck	=>	mcb3_dram_ck,		
		mcb3_dram_ck_n	=>	mcb3_dram_ck_n,		
		--ONCHIP PORTS:		
		--channel-0			
		channel0_wb_data_o	=>	channel0_wb_data_o, --vector(31 downto 0)
		channel0_wb_data_i	=>	channel0_wb_data_i, --vector(31 downto 0)
		channel0_wb_addr_i	=>	channel0_wb_addr_i, --vector(25 downto 0)
		channel0_wb_cyc_i	=>	channel0_wb_cyc_i,		
		channel0_wb_stb_i	=>	channel0_wb_stb_i,		
		channel0_wb_wr_i	=>	channel0_wb_wr_i,		
		channel0_wb_ack_o	=>	channel0_wb_ack_o,		
		channel0_wb_clk_i	=>	channel0_wb_clk_i,		
		--channel-1			
		channel1_wb_data_o	=>	channel1_wb_data_o, --vector(31 downto 0)
		channel1_wb_data_i	=>	channel1_wb_data_i, --vector(31 downto 0)
		channel1_wb_addr_i	=>	channel1_wb_addr_i, --vector(25 downto 0)
		channel1_wb_cyc_i	=>	channel1_wb_cyc_i,		
		channel1_wb_stb_i	=>	channel1_wb_stb_i,		
		channel1_wb_wr_i	=>	channel1_wb_wr_i,		
		channel1_wb_ack_o	=>	channel1_wb_ack_o,		
		channel1_wb_clk_i	=>	channel1_wb_clk_i,		
		--channel-2			
		channel2_wb_data_o	=>	channel2_wb_data_o, --vector(31 downto 0)
		channel2_wb_data_i	=>	channel2_wb_data_i, --vector(31 downto 0)
		channel2_wb_addr_i	=>	channel2_wb_addr_i, --vector(25 downto 0)
		channel2_wb_cyc_i	=>	channel2_wb_cyc_i,		
		channel2_wb_stb_i	=>	channel2_wb_stb_i,		
		channel2_wb_wr_i	=>	channel2_wb_wr_i,		
		channel2_wb_ack_o	=>	channel2_wb_ack_o,		
		channel2_wb_clk_i	=>	channel2_wb_clk_i,		
		--channel-3			
		channel3_wb_data_o	=>	channel3_wb_data_o, --vector(31 downto 0)
		channel3_wb_data_i	=>	channel3_wb_data_i, --vector(31 downto 0)
		channel3_wb_addr_i	=>	channel3_wb_addr_i, --vector(25 downto 0)
		channel3_wb_cyc_i	=>	channel3_wb_cyc_i,		
		channel3_wb_stb_i	=>	channel3_wb_stb_i,		
		channel3_wb_wr_i	=>	channel3_wb_wr_i,		
		channel3_wb_ack_o	=>	channel3_wb_ack_o,		
		channel3_wb_clk_i	=>	channel3_wb_clk_i,		
		--System	signals:		
		x25m_clk	=>	x25m_clk,		
		reset	=>	reset	
	);	
		
		
	Inst_s6bfip_pcie: s6bfip_pcie PORT MAP(			
		pci_exp_txp	=>	pci_exp_txp,		
		pci_exp_txn	=>	pci_exp_txn,		
		pci_exp_rxp	=>	pci_exp_rxp,		
		pci_exp_rxn	=>	pci_exp_rxn,		
		sys_clk_n	=>	sys_clk_n,
		sys_clk_p	=>	sys_clk_p,			
		sys_reset_n	=>	sys_reset_n,		
		--ON	CHIP		PORTS:
		--DATA	BUS		(wishbone):			
		pcieif_wb_data_o	=>	pcieif_wb_data_o,	--vector(31 downto 0)	
		pcieif_wb_data_i	=>	pcieif_wb_data_i,	--vector(31 downto 0)	
		pcieif_wb_addr_o	=>	pcieif_wb_addr_o,	--vector(25 downto 0)	
		pcieif_wb_cyc_o	=>	pcieif_wb_cyc_o,		
		pcieif_wb_stb_o	=>	pcieif_wb_stb_o,		
		pcieif_wb_wr_o	=>	pcieif_wb_wr_o,		
		pcieif_wb_ack_i	=>	pcieif_wb_ack_i,		
		pcieif_wb_clk_o	=>	pcieif_wb_clk_o,	--62.5MHz				
		--AURORA control		bus:			
		aurora_config_wb_data_o	=>	aurora_config_wb_data_o,	--vector(31 downto 0)			
		aurora_config_wb_data_i	=>	aurora_config_wb_data_i,	--vector(31 downto 0)			
		aurora_config_wb_addr_i	=>	aurora_config_wb_addr_i,	--vector(4 downto 0)			
		aurora_config_wb_cyc_i	=>	aurora_config_wb_cyc_i,	
		aurora_config_wb_stb_i	=>	aurora_config_wb_stb_i,	
		aurora_config_wb_wr_i	=>	aurora_config_wb_wr_i,	
		aurora_config_wb_ack_o	=>	aurora_config_wb_ack_o,	
		aurora_config_wb_clk_i	=>	aurora_config_wb_clk_i,	
		--DSP firmware loader bus:		
		dsp_instr_wb_data_o	=>	dsp_instr_wb_data_o,	--vector(31 downto 0)
		dsp_instr_wb_data_i	=>	dsp_instr_wb_data_i,	--vector(31 downto 0)
		dsp_instr_wb_addr_o	=>	dsp_instr_wb_addr_o,	--vector(15 downto 0)
		dsp_instr_wb_cyc_o	=>	dsp_instr_wb_cyc_o,	
		dsp_instr_wb_stb_o	=>	dsp_instr_wb_stb_o,	
		dsp_instr_wb_wr_o	=>	dsp_instr_wb_wr_o,	
		dsp_instr_wb_ack_i	=>	dsp_instr_wb_ack_i,	
		dsp_instr_wb_clk_o	=>	dsp_instr_wb_clk_o,	
		--ADC control		
		adc_overshoot_detected	=>	adc_overshoot_detected,	
		pga_gain	=>	pga_gain,	--vector(5 downto 0)			
		adc_triggerlevel	=>	adc_triggerlevel,	--vector(7 downto 0)
		adc_start	=>	adc_start,	--edge	sensitive				
		adc_finished	=>	adc_finished,				
		--DSP control		
		dsp_start_boot_frompcie	=>	dsp_start_boot_frompcie,				
		dsp_reset_control_frompcie	=>	dsp_reset_control_frompcie,	
		dsp_irq_frompcie	=>	dsp_irq_frompcie,	
		dsp_nmi_frompcie	=>	dsp_nmi_frompcie,	
		dsp_flag_to_pcie	=>	dsp_flag_to_pcie,	--vector(1 downto 0)		
		dsp_flag_from_pcie	=>	dsp_flag_from_pcie,	--vector(1 downto 0)	
		--OTHER:			
		pcie_resetout	=>	pcie_resetout				
	);	
		
		
		
		
		
	

	-- MAIN LOGIC: -----------------------------------------------------------------








-- -------- END OF FILE -----------------------------------------------------------
end Behavioral;

