----------------------------------------------------------------------------------
-- Company: Home, hobby project
-- Engineer: Istvan Nagy
-- 
-- Create Date:    15:40:36 05/30/2010 
-- Design Name:  Spartan6-Blackfin-Board
-- Module Name:    s6bf_board_top - Behavioral 
-- Project Name: 
-- Target Devices: XC6SLX45T
-- Tool versions: ISE 11.4
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
--  ===========SYSTEM:=============
--
--  DDR2_chip (64Mbytes,500MT/s,x16)
--    |
-- [MEM.CO.]
--  | | | |_____ADC_INTERFACE<--ADS5517 A/D Converter (200MSPS,LVDS)
--  | | |_______DSP_INTERFACE<-->ADSP-BF544 DSP, (500MHz)
--  | |_________AURORA_INTERFACE<-->RJ45 connector, (AC coupled, 2lanes, 2.5Gbps)
--  |___________PCIE-ENDPOINT,BAR1<-->card_edge (x1, PCIe 1.1)
--
-- PCIE endpoint BAR0 usage:
--  address 0...64kB: Control and status registers for the other interfaces.
--  address 64kB...128kB: DSP firmware FIFO, 64KB storage.
--
-- ADDRESSES ARE BYTE ADDRESSES. so the dword peripherals must ignore addr[1:0] 
--
-- Control&Status Registers BAR0 Address Map (0000-1FFFF: LSB-bytes only):
-- -------------------------------------
--  addr   name            bit7   bit6   bit5   bit4   bit3   bit2   bit1   bit0
--  00h    PGA_gain_reg    -      -      b5     b4     b3     b2     b1     b0
--  04h    ADC_cont_reg    start  finish oversh -      -      -      -      -
--  08h    ADC_trigg_reg   b7     b6     b5     b4     b3     b2     b1     b0
--  0Ah    DSP_contr_reg   boot   reset  irq    nmi    -      -      -      -
--  10h    DSP_flag_reg    -      -      -      -      out1   out0   in1    in0
--  100h to 17F: Aurora config/status registers
--
-- 10000
--  to     DSP_instr_FIFO
-- 1FFFF
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity s6bf_board_top is
    
	 Port ( --TOPLEVEL PORTS-----------------------
			 --ADC INTERFACE---
			  ADC_CLK_REFOUT_P : out    std_logic; --set LVDS+Rs --200MHz
			  ADC_CLK_REFOUT_N  : out    std_logic; --set LVDS+Rs			 
			  ADC_CLK_IN_P   : in    std_logic; --set LVDS+Rp, use GCLK-pin
			  ADC_CLK_IN_N   : in    std_logic; --set LVDS+Rp, use GCLK-pin
			  ADC_DATA_IN_P   : in    std_logic_vector(5 downto 0); --set LVDS+Rp, same bufio2-region
			  ADC_DATA_IN_N   : in    std_logic_vector(5 downto 0); --set LVDS+Rp, same bufio2-region
			  ADC_OE : out std_logic;
			  ADC_OVR : in std_logic;
			  ADC_SDATA : out std_logic;
			  ADC_SCLK : out std_logic;
			  ADC_SEN : out std_logic;
			  ADC_RESET : out std_logic;
			  PGA_DATA_out  : out    std_logic_vector(5 downto 0); 
			  PGA_STROBE : out std_logic; 
			  --DSP INTERFACE:
			  dsp_reset_n : out  STD_LOGIC;
			  dsp_refclk :   out  STD_LOGIC;	
			  dsp_ioclk: in  STD_LOGIC;  --to a GCLK pin!
			  dsp_bmode : out  STD_LOGIC_VECTOR (3 downto 0);
			  dsp_dmar1_n : out  STD_LOGIC;
			  dsp_hwait : in  STD_LOGIC;
			  dsp_ams_n : in  STD_LOGIC_VECTOR (3 downto 0);
			  dsp_data : inout  STD_LOGIC_VECTOR (15 downto 0);
			  dsp_abe_n :in  STD_LOGIC_VECTOR (1 downto 0);
			  dsp_awe_n : in  STD_LOGIC;  --to a GCLK pin!
			  dsp_aoe_n : in  STD_LOGIC;
			  dsp_are_n : in  STD_LOGIC;  --to a GCLK pin!
			  dsp_ardy : out  STD_LOGIC;
			  dsp_addr : in STD_LOGIC_VECTOR (25 downto 0);
			  dsp_flag_from_dsp : in  STD_LOGIC_VECTOR (1 downto 0);
			  dsp_flag_to_dsp : out  STD_LOGIC_VECTOR (1 downto 0);
			  dsp_uart_tx : out  STD_LOGIC;
			  dsp_uart_rx : in  STD_LOGIC;
			  dsp_interrupt : out  STD_LOGIC;
			  dsp_nmi : out  STD_LOGIC;
			  dsp_hostif_data :inout  STD_LOGIC_VECTOR (7 downto 0);
			  dsp_hostif_ce_n : out  STD_LOGIC;
			  dsp_hostif_rd_n : out  STD_LOGIC;
			  dsp_hostif_wr_n : out  STD_LOGIC;
			  dsp_hostif_addr : out  STD_LOGIC_VECTOR (1 downto 0);
			  dsp_hostif_ack : in  STD_LOGIC;
			  --AURORA INTERFACE:
           aurora_RXP  : in std_logic_vector(0 to 1);
           aurora_RXN  : in std_logic_vector(0 to 1);
           aurora_TXP  : out std_logic_vector(0 to 1);
           aurora_TXN  : out std_logic_vector(0 to 1);
			  --DDR MEMORY PINS:
			  mcb3_dram_dq        : inout  std_logic_vector(15 downto 0);
			  mcb3_dram_a         : out std_logic_vector(12 downto 0);
			  mcb3_dram_ba        : out std_logic_vector(1 downto 0);
			  mcb3_dram_ras_n     : out std_logic;
			  mcb3_dram_cas_n     : out std_logic;
			  mcb3_dram_we_n      : out std_logic;
			  mcb3_dram_odt       : out std_logic;
			  mcb3_dram_cke       : out std_logic;
			  mcb3_dram_dm        : out std_logic;
			  mcb3_dram_udqs      : inout  std_logic;
			  mcb3_dram_udqs_n    : inout  std_logic;
			  mcb3_rzq            : inout  std_logic;
			  mcb3_zio            : inout  std_logic;
			  mcb3_dram_udm       : out std_logic;
			  mcb3_dram_dqs       : inout  std_logic;
			  mcb3_dram_dqs_n     : inout  std_logic;
			  mcb3_dram_ck        : out std_logic;
			  mcb3_dram_ck_n      : out std_logic;		
			  --PCI-EXPRESS INTERFACE:
			  pci_exp_txp             : out std_logic;
			  pci_exp_txn             : out std_logic;
			  pci_exp_rxp             : in  std_logic;
			  pci_exp_rxn             : in  std_logic;	 
			  sys_clk_n                 : in  std_logic;
			  sys_clk_p                 : in  std_logic;
			  sys_reset_n             : in  std_logic;					  
			  -- OTHER:
			  x25m_clkin : in  STD_LOGIC;
			  led0 : out  STD_LOGIC;
			  led1 : out  STD_LOGIC;
			  led2 : out  STD_LOGIC;
			  led3 : out  STD_LOGIC
			  );
end s6bf_board_top;

-- --------ARCHITECTURE BEGINS ----------------------------------------------------
architecture Behavioral of s6bf_board_top is


	-- COMPONENT DECLARATIONS (introducing the IPs) --------------------------------




-- --------ARCHITECTURE BODY BEGINS -----------------------------------------------
begin




	-- COMPONENT INSTALLATIONS (connecting the IPs to local signals) ---------------
	
		
		
		
	

	-- MAIN LOGIC: -----------------------------------------------------------------








-- -------- END OF FILE -----------------------------------------------------------
end Behavioral;

